//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

#import "ViewController.h"
#import "KakaoSDKObjcSample-Swift.h"

// MARK: - tableview helper =============================================================================================

@interface Row : NSObject
typedef void (^SDKSampleRowAction)(void);

@property (strong, nonatomic) NSString* title;
@property (copy, nonatomic) SDKSampleRowAction action;

- (id)initWithTitle:(NSString*)title action:(SDKSampleRowAction)action;
@end

@implementation Row
- (instancetype)initWithTitle:(NSString*)title action:(SDKSampleRowAction)action {
    self = [super init];
    if (self) {
        _title = title;
        _action = action;
    }
    
    return self;
}
@end


@interface SectionOfRow : NSObject
@property (strong, nonatomic) NSString* header;
@property (strong, nonatomic) NSArray<Row*> *rows;

- (instancetype)initWithHeader:(NSString*)header rows:(NSArray<Row*> *)rows;
- (instancetype)initWithOriginal:(SectionOfRow*)original rows:(NSArray<Row*> *)rows;
@end

@implementation SectionOfRow
- (instancetype)initWithHeader:(NSString*)header rows:(NSArray<Row*> *)rows {
    self = [super init];
    if (self) {
        _header = header;
        _rows = rows;
    }
    
    return self;
}

- (instancetype)initWithOriginal:(SectionOfRow*)original rows:(NSArray*)rows {
    self = [self initWithHeader:original.header rows:original.rows];
    if (self) {
        _rows = rows;
    }
    
    return self;
}
@end



@interface ViewController ()  <UITableViewDelegate, UITableViewDataSource>
//@property (strong, nonatomic) UIView* backgoundView;
@property (strong, nonatomic) UILayoutGuide* safeArea;
@property (strong, nonatomic) UITableView* tableView;
@property (strong, nonatomic) NSArray<SectionOfRow*> *sections;
@end

@implementation ViewController

- (void)loadView {
    [super loadView];
    
    self.tableView = [[UITableView alloc] initWithFrame:self.view.frame style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    
    [self.view addSubview:self.tableView];
    
    self.safeArea = self.view.layoutMarginsGuide;
    self.tableView.translatesAutoresizingMaskIntoConstraints = NO;
    [self.tableView.leadingAnchor constraintEqualToAnchor:self.safeArea.leadingAnchor].active = YES;
    [self.tableView.trailingAnchor constraintEqualToAnchor:self.safeArea.trailingAnchor].active = YES;
    [self.tableView.topAnchor constraintEqualToAnchor:self.safeArea.topAnchor].active = YES;
    [self.tableView.bottomAnchor constraintEqualToAnchor:self.safeArea.bottomAnchor].active = YES;
    
    if (@available(iOS 15.0, *)) {
        [[UITableView appearance] setSectionHeaderTopPadding:0.0f];
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = UIColor.whiteColor;
    
    [self.tableView registerClass:UITableViewCell.self forCellReuseIdentifier: @"Cell"];
    self.tableView.backgroundColor = UIColor.whiteColor;
//    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    
    [self setupTitle];
    [self setMenu];
}

- (void)setupTitle {
    self.navigationItem.title = @"Objc SDK Sample";
    self.navigationController.navigationBar.titleTextAttributes = [NSDictionary dictionaryWithObject:[UIColor blackColor]
                                                                                              forKey:NSForegroundColorAttributeName];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellId = @"Cell";
        
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellId];
    
    Row* row = self.sections[indexPath.section].rows[indexPath.row];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc]initWithStyle:
                UITableViewCellStyleDefault reuseIdentifier:cellId];
    }
    cell.backgroundColor = UIColor.whiteColor;
    [cell.textLabel setText:[NSString stringWithFormat:@"%@", row.title]];
    cell.textLabel.adjustsFontSizeToFitWidth = YES;
    cell.textLabel.textColor = UIColor.blackColor;
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (self.sections != NULL && [self.sections count] > 0) {
        return self.sections[section].rows.count;
    }
    else {
        return 0;
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    if (self.sections != NULL && [self.sections count] > 0) {
        NSLog(@"sections count: %ld", self.sections.count);
        return self.sections.count;
    }
    else {
        return 0;
    }
}

- (NSString*)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if (self.sections != NULL && [self.sections count] > 0) {
        return self.sections[section].header;
    }
    else {
        return NULL;
    }
}

- (void)tableView:(UITableView *)tableView willDisplayHeaderView:(UIView *)view forSection:(NSInteger)section
{
    view.tintColor = [UIColor grayColor];

    UITableViewHeaderFooterView *header = (UITableViewHeaderFooterView *)view;
    [header.textLabel setTextColor:[UIColor blackColor]];

    // Another way to set the background color
    // Note: does not preserve gradient effect of original header
    // header.contentView.backgroundColor = [UIColor blackColor];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 30;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (self.sections != NULL && self.sections.count > 0) {
        Row* row = self.sections[indexPath.section].rows[indexPath.row];
        if (row.action != NULL) {
            row.action();
        }
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1 * 1), dispatch_get_main_queue(), ^{
            [self.tableView deselectRowAtIndexPath:indexPath animated:NO];
        });
    }
}



- (void)setMenu {
    self.sections = @[
        [[SectionOfRow alloc] initWithHeader:@"AUTH" rows: @[
            [[Row alloc] initWithTitle:@"getToken()" action:^{
                NSDictionary* tokenInfo = [[KakaoSDKAuthWrapper shared] getToken];
                NSLog(@"tokenInfo: %@", tokenInfo);
            }]
            
        ]],
        [[SectionOfRow alloc] initWithHeader:@"USER" rows: @[
            [[Row alloc] initWithTitle:@"loginWithKakaoTalk()" action:^{
            
                if ([KakaoSDKUserWrapper isKakaoTalkLoginAvailable]) {
                    [[KakaoSDKUserWrapper shared] loginWithKakaoTalkWithChannelPublicIds:NULL
                                                                            serviceTerms:NULL
                                                                                   nonce:NULL
                                                                              completion:^(NSDictionary<NSString *,id> *response, NSError *error) {
                        if (error != NULL) {
                            NSLog(@"error: %@", error);
                        }
                        else {
                            NSLog(@"tokenInfo: %@", response);
                        }
                    }];
                }
            }],
            [[Row alloc] initWithTitle:@"loginWithKakaoAccount()" action:^{
                [[KakaoSDKUserWrapper shared] loginWithKakaoAccountWithPrompts:NULL loginHint:NULL completion:^(NSDictionary<NSString *,id> *response, NSError *error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"tokenInfo: %@", response);
                    }
                }];
            }],
            [[Row alloc] initWithTitle:@"logout()" action:^{
                [[KakaoSDKUserWrapper shared] logoutWithCompletion:^(NSError * _Nullable error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                }];
            }],
            [[Row alloc] initWithTitle:@"me()" action:^{
            
                [[KakaoSDKUserWrapper shared] meWithPropertyKeys:NULL secureResource:NULL completion:^(NSDictionary<NSString *,id> *response, NSError *error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"user: %@", response);
                    }
                }];
            }],
            [[Row alloc] initWithTitle:@"updateProfile()" action:^{
            [[KakaoSDKUserWrapper shared] updateProfileWithProperties:@{@"custom_key":@"update_profile_test"} completion:^(NSError * _Nullable error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                }];
            }],
            [[Row alloc] initWithTitle:@"accessTokenInfo()" action:^{
                [[KakaoSDKUserWrapper shared] accessTokenInfoWithCompletion:^(NSDictionary<NSString *,id> *response, NSError *error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"tokenInfo: %@", response);
                    }
                }];
            }]
        ]]
        ,
        [[SectionOfRow alloc] initWithHeader:@"TALK" rows: @[
            [[Row alloc] initWithTitle:@"profile()" action:^{
                [[KakaoSDKTalkWrapper shared] profileWithCompletion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"profile: %@", response);
                    }
                }];
                
            }]
        ]],
        [[SectionOfRow alloc] initWithHeader:@"FRIEND" rows: @[
            [[Row alloc] initWithTitle:@"friends()" action:^{
                [[KakaoSDKFriendWrapper shared] selectFriendsWithCompletion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"selected friends: %@", response);
                    }
                }];
                
            }],
            [[Row alloc] initWithTitle:@"friendsPopup()" action:^{
                [[KakaoSDKFriendWrapper shared] selectFriendsPopupWithCompletion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"selected friends: %@", response);
                    }
                }];
                
            }]
        ]],
        [[SectionOfRow alloc] initWithHeader:@"STORY" rows: @[
            [[Row alloc] initWithTitle:@"profile()" action:^{
                [[KakaoSDKStoryWrapper shared] profileWithSecureResource:YES completion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"profile: %@", response);
                    }
                }];
                
            }]
        ]],
        [[SectionOfRow alloc] initWithHeader:@"SHARE" rows: @[
            [[Row alloc] initWithTitle:@"shareDefault()" action:^{
            
                NSString* feedTemplateJsonString = SampleData.feedTemplateJson;
                [[KakaoSDKShareWrapper shared] shareDefaultWithTemplateJsonString:feedTemplateJsonString serverCallbackArgs:NULL completion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"sharingResult: %@", response);
                        
                        NSURL* url = [NSURL URLWithString:[NSString stringWithFormat:@"%@", response[@"url"]]];
                        if (url != NULL) {
                            if ([[UIApplication sharedApplication] canOpenURL:url]) {
                                [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                            }
                        }
                    }
                }];
            }]
            ,
            [[Row alloc] initWithTitle:@"shareScrap()" action:^{
                [[KakaoSDKShareWrapper shared] shareScrapWithRequestUrl:@"https://developers.kakao.com" templateId:SampleData.templateIdLink templateArgs:NULL serverCallbackArgs:NULL completion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"sharingResult: %@", response);
                        
                        NSURL* url = [NSURL URLWithString:[NSString stringWithFormat:@"%@", response[@"url"]]];
                        if (url != NULL) {
                            if ([[UIApplication sharedApplication] canOpenURL:url]) {
                                [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                            }
                        }
                    }
                }];
            }]
            ,
            [[Row alloc] initWithTitle:@"shareCustom()" action:^{
                [[KakaoSDKShareWrapper shared] shareCustomWithTemplateId:SampleData.templateIdLink templateArgs:@{@"title":@"제목입니다.", @"description":@"설명입니다."} serverCallbackArgs:NULL completion:^(NSDictionary<NSString *,id> * response, NSError * error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"sharingResult: %@", response);
                        
                        NSURL* url = [NSURL URLWithString:[NSString stringWithFormat:@"%@", response[@"url"]]];
                        if (url != NULL) {
                            if ([[UIApplication sharedApplication] canOpenURL:url]) {
                                [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                            }
                        }
                    }
                }];            
            }]
            ,
            [[Row alloc] initWithTitle:@"imageUpload(imageUploadWithImage:secureResource:completion:)" action:^{
            
                UIImage* image = [UIImage imageNamed:@"sample1"];
                
                [[KakaoSDKShareWrapper shared] imageUploadWithImage:image secureResource:YES completion:^(NSDictionary<NSString *,id> * _Nullable response, NSError * _Nullable error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"imageUploadResult: %@", response);
                    }
                }];
            }]
            ,
            [[Row alloc] initWithTitle:@"imageScrap(imageScrapWithImageUrl:secureResource:completion:)" action:^{
            
                NSURL* url = [NSURL URLWithString:@"https://t1.kakaocdn.net/kakaocorp/Service/KakaoTalk/pc/slide/talkpc_theme_01.jpg"];
                [[KakaoSDKShareWrapper shared] imageScrapWithImageUrl:url secureResource:YES completion:^(NSDictionary<NSString *,id> * _Nullable response, NSError * _Nullable error) {
                    if (error != NULL) {
                        NSLog(@"error: %@", error);
                    }
                    else {
                        NSLog(@"imageUploadResult: %@", response);
                    }
                }];
            }]
        ]],
        [[SectionOfRow alloc] initWithHeader:@"NAVI" rows: @[
            [[Row alloc] initWithTitle:@"share(destination) - katec" action:^{
                NSURL* shareUrl = [[KakaoSDKNaviWrapper shared] shareUrlForKatec];
                            
                NSLog(@"shareUrl: %@", shareUrl);
                if (shareUrl != NULL) {
                    if ([[UIApplication sharedApplication] canOpenURL:shareUrl]) {
                        [[UIApplication sharedApplication] openURL:shareUrl options:@{} completionHandler:nil];
                    }
                    else {
                        [[UIApplication sharedApplication] openURL:KakaoSDKNaviWrapper.webNaviInstallUrl options:@{} completionHandler:nil];
                    }
                }
            }],
            [[Row alloc] initWithTitle:@"share(destination) - wgs84" action:^{
                NSURL* shareUrl = [[KakaoSDKNaviWrapper shared] shareUrlForWgs84];
                            
                NSLog(@"shareUrl: %@", shareUrl);
                if (shareUrl != NULL) {
                    if ([[UIApplication sharedApplication] canOpenURL:shareUrl]) {
                        [[UIApplication sharedApplication] openURL:shareUrl options:@{} completionHandler:nil];
                    }
                    else {
                        [[UIApplication sharedApplication] openURL:KakaoSDKNaviWrapper.webNaviInstallUrl options:@{} completionHandler:nil];
                    }
                }
            }],
            [[Row alloc] initWithTitle:@"navigate(destination) - katec" action:^{
                NSURL* navigateUrl = [[KakaoSDKNaviWrapper shared] navigateUrlForKatec];
                            
                NSLog(@"navigateUrl: %@", navigateUrl);
                if (navigateUrl != NULL) {
                    if ([[UIApplication sharedApplication] canOpenURL:navigateUrl]) {
                        [[UIApplication sharedApplication] openURL:navigateUrl options:@{} completionHandler:nil];
                    }
                    else {
                        [[UIApplication sharedApplication] openURL:KakaoSDKNaviWrapper.webNaviInstallUrl options:@{} completionHandler:nil];
                    }
                }
            }],
            [[Row alloc] initWithTitle:@"navigate(destination) - wgs84" action:^{
                NSURL* navigateUrl = [[KakaoSDKNaviWrapper shared] shareUrlForWgs84];
                            
                NSLog(@"navigateUrl: %@", navigateUrl);
                if (navigateUrl != NULL) {
                    if ([[UIApplication sharedApplication] canOpenURL:navigateUrl]) {
                        [[UIApplication sharedApplication] openURL:navigateUrl options:@{} completionHandler:nil];
                    }
                    else {
                        [[UIApplication sharedApplication] openURL:KakaoSDKNaviWrapper.webNaviInstallUrl options:@{} completionHandler:nil];
                    }
                }
            }]
        ]],
    ];
}
@end




